#!/bin/bash
#https://www.drupal.org/node/118759
logdir=$HOME/check-websites/geen-backup
icon='--window-icon=emblem-favorite'
icon=""
YAD="yad $icon --center --on-top"
if command -v setwmc > /dev/null ; then
   YAD="setwmc check-website check-website $YAD"
fi
mkdir -p "$logdir" || exit
cd "$logdir" || exit
usegui=""
if [ -z "$1" ] ; then
   usegui=yes
fi
usage()
{
   echo "Usage: $0"' [-c "words to check in content" [-f "words to check in names"] [-n loginname] url'
   echo "example: $0"' -c "viagra jukebox" -f "viagra" www.ratrabbit.nl/ratrabbit'
}
cachedir=$HOME/.cache/check-website
mkdir -p $cachedir
if [ "$usegui" ] ; then
   website=""
   if [ -f $cachedir/website ] ; then
      website="`cat $cachedir/website|head -n 1`"
   fi
   website=`$YAD --title "Which website?" --entry --text "Enter url of website" --entry-text="$website"`
   if [ "$?" != 0 ] ; then
      exit 1
   fi
   website=`echo "$website" | sed 's/^[sed 	]*//'`
   if [ -z "$website" ] ; then
      exit 1
   fi
   zenitytitle="check website $website"

   level=8
   if [ -f $cachedir/level ] ; then
      level="`cat $cachedir/level|head -n 1`"
   fi
   level=`$YAD --title "Depth of search" --entry \
      --entry-text="$level" \
      --text "Maximal depth, for example 8"`
   if [ "$?" != 0 ]; then
      exit 1
   fi
   level=${level//[^0-9]/}

   user=""
   if [ -f $cachedir/user ] ; then
      user="`cat $cachedir/user|head -n 1`"
   fi
   user=`$YAD --title "Login?" --entry \
      --entry-text="$user" \
      --text "Login, blank for no login"`
   if [ "$?" != 0 ] ; then
      exit 1
   fi
   user=`echo "$user" | sed 's/^[ 	]*//'`

   contentcheck="viagra"
   if [ -f $cachedir/contentcheck ] ; then
      contentcheck="`cat $cachedir/contentcheck|head -n 1`"
   fi
   contentcheck=`$YAD --title "Dirty words" --entry \
      --entry-text="$contentcheck" \
      --text "Check website content on the following words:"`
   if [ "$?" != 0 ] ; then
      exit 1
   fi
   contentcheck=`echo "$contentcheck" | sed 's/^[ 	]*//'`

   filecheck="viagra"
   if [ -f $cachedir/filecheck ] ; then
      filecheck="`cat $cachedir/filecheck|head -n 1`"
   fi
   filecheck=`$YAD --title "Dirty filenames" --entry \
      --entry-text="$filecheck" \
      --text "Check website file names on the following words:"`
   if [ "$?" != 0 ] ; then
      exit 1
   fi
   filecheck=`echo "$filecheck" | sed 's/^[ 	]*//'`

   echo "$filecheck"    > $cachedir/filecheck
   echo "$website"      > $cachedir/website
   echo "$contentcheck" > $cachedir/contentcheck
   echo "$user"         > $cachedir/user
   echo "$level"        > $cachedir/level

else
   parms=$( getopt -o c:f:n:l: -- "$@" ) || { usage; exit; }
   echo TD: parms: $parms
   eval set -- "$parms"
   for i in $@ ; do
      echo TD: @: $i
   done
   user=""
   level=8
   while [ "$1" ] ; do
      case "$1" in
	 -c)
	    shift
	    contentcheck="$1"
	    shift
	    ;;
	 -n)
	    shift
	    user="$1"
	    shift
	    ;;
	 -f)
	    shift
	    filecheck="$1"
	    shift
	    ;;
	 -l)
	    shift
	    level="$1"
	    shift
	    ;;
	 --)
	    shift
	    website="$1"
	    shift
	    ;;
      esac
   done
fi
echo TD: contentcheck $contentcheck
echo TD: filecheck $filecheck
echo TD: user $user
echo TD: level $level
echo TD: website $website

if [ -z "$website" ] ; then
   exit
fi

website_full="$website"
website=`echo ${website#*:/} | sed 's|^/*||'`
website_base=${website%%/*}

wgetname=`mktemp -u wgetXXXXX`
cookies=`mktemp`

out1()
{
   if [ "$usegui" ] ; then
      ( echo 20
      echo '#Please wait ...'
      cat ) | \
      $YAD \
	 --title="$zenitytitle" \
	 --progress \
	 --pulsate \
	 --auto-close \
	 --text="downloading $website ..."
      if [ $? -ne 0 ] ; then
	 pkill -f $wgetname
	 touch stoppit
      fi
   else
      cat
   fi
}

out2()
{
   if [ "$usegui" ] ; then
      cat > tmpfile.txt
      gedit --new-window tmpfile.txt
   else
      cat
   fi
}

webdir="${website%%/*}-logs" 
echo TD: website: $website
echo TD: webdir: $webdir
echo TD: website_base: $website_base
rm -rf "$webdir"
mkdir "$webdir" || exit
cd "$webdir" || exit

x="/user/password?name=$user"

if [ "$user" ] ; then
   pwok=no
   while [ "$pwok" != yes ] ; do
      if [ "$usegui" ] ; then
	 pw=`$YAD --password --text "Password for $user"`
	 if [ -z "$pw" ] ; then
	    rm -f $cookies
	    exit
	 fi
      else
	 echo -n "Password for $user:"
	 read -s pw
      fi
      wget \
	 --no-check-certificate \
	 --keep-session-cookies \
	 --save-cookies $cookies \
	 --load-cookies $cookies \
	 --post-data="name=$user&pass=$pw&op=Log%20in&form_id=user_login" \
	 -O pwcheck \
	 "$website/user" 
      if grep -q "$x" pwcheck ; then
	 if [ $usegui ] ; then
	    a="Wrong password for $user, try again"
	    $YAD --error --text="$a"
	 else
	    echo "$a"
	 fi
      else
	 pwok=yes
      fi
   done
fi

bash -c "exec -a $wgetname wget \
   -e robots=off \
   --no-check-certificate \
   --mirror \
   --level="$level" \
   --no-cache \
   --no-verbose \
   --convert-links \
   --adjust-extension \
   --reject-regex '.*/node/[0-9]*' \
   --reject logout,signout,delete,remove,exit,edit \
   --load-cookies $cookies  \
   --keep-session-cookies \
   --save-cookies $cookies \
   \"$website_full\"" 2>&1 | \
   egrep -B 1 "ERROR|unable to resolve" | tee log | out1

# for older wget (as on lisa)
# replace the --reject-regex line with:
#   --reject '[0-9]*' \

if [ -e stoppit ] ; then
   echo "Forced end"
   rm -f $cookies
   exit 1
fi

(
grep -v "ERROR" log | grep -ve "--" | sed 's/:$//' > problem-pages
echo "Problem pages:" | tee results
echo "==============" | tee -a results
cat problem-pages | 
while read a ; do 
   echo "$a" 
   grep -rl -e "$a" "$website_base" | sed 's/\.html$//' | sed 's/^/found in: /'
done | tee -a results

for word in $filecheck ; do
   (
      echo
      echo "Files with '$word' in the name:"
      echo "=============================="
      find "$website_base" -iname "*$word*"
   ) | tee -a results
done

for word in $contentcheck ; do
   (
      echo
      echo "Text files (possibly without the .html suffix) containing the word '$word':"
      echo "=============================="
      grep -ri "$word" "$website_base"  | grep -v Binary
   ) | tee -a results
done

echo 
echo "All results in folder $logdir/$webdir"
echo 
echo "Copy of website              : $website"
echo "Problem pages                : problem-pages (`wc -l < problem-pages`)"
echo "Problem pages plus locations : results"
) 2>&1 | out2
